﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectSeaplane.Entities;

namespace ProjectSeaplane.Drawings;
public class DrawingPlane
{
    public EntityPlane? EntityPlane { get; protected set; }

    private int? _pictureWidth;

    private int? _pictureHeight;

    protected int? _startPosX;

    protected int? _startPosY;

    private readonly int _drawingPlaneWidth = 190;

    private readonly int _drawingPlaneHeight = 70;

    public int? GetPosX => _startPosX;

    public int? GetPosY => _startPosY;

    public int GetWidth => _drawingPlaneWidth;

    public int GetHeight => _drawingPlaneHeight;
    
    private DrawingPlane()
    {
        _pictureHeight = null;
        _pictureWidth = null;
        _startPosX = null;
        _startPosY = null;
    }

    public DrawingPlane(int speed, double weight, Color bodyColor) : this()
    {
        EntityPlane = new EntityPlane(speed, weight, bodyColor);
    }

    protected DrawingPlane(int drawingSeaplaneWidth, int drawingSeaplaneHeight) : this()
    {
        _drawingPlaneWidth = drawingSeaplaneWidth;
        _drawingPlaneHeight = drawingSeaplaneHeight;
    }

    public DrawingPlane(EntityPlane plane) : this()
    {
        EntityPlane = new EntityPlane(plane.Speed, plane.Weight, plane.BodyColor);
    }

    public bool SetPictureSize(int width, int height)
    {
        if (_drawingPlaneWidth > width || _drawingPlaneHeight > height)
        {
            return false;
        }
        _pictureWidth = width;
        _pictureHeight = height;

        if (_startPosX.HasValue || _startPosY.HasValue)
        {

            if (_startPosX + _drawingPlaneWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth - _drawingPlaneWidth;
            }
            else if (_startPosX < 0) _startPosX = 0;
            if (_startPosY + _drawingPlaneHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight - _drawingPlaneHeight;
            }
            else if (_startPosY < 0) _startPosY = 0;
        }
        return true;
    }

    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }

        if (x < 0)
        {
            x = 0;
        }
        else if (x - _drawingPlaneWidth >= _pictureWidth.Value)
        {
            x = _pictureWidth.Value - _drawingPlaneWidth - _drawingPlaneWidth;
        }

        if (y < 0)
        {
            y = 0;
        }
        else if (y - _drawingPlaneHeight >= _pictureHeight)
        {
            y = _pictureHeight.Value - _drawingPlaneHeight - _drawingPlaneHeight;
        }

        _startPosY = y;
        _startPosX = x;
    }

    public bool MoveTransport(DirectionType direction)
    {
        if (EntityPlane == null || !_startPosX.HasValue || !_startPosY.HasValue || !_pictureWidth.HasValue || !_pictureHeight.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityPlane.Step >= 0)
                {
                    _startPosX -= (int)EntityPlane.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityPlane.Step >= 0)
                {
                    _startPosY -= (int)EntityPlane.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityPlane.Step <= _pictureWidth.Value - _drawingPlaneWidth)
                {
                    _startPosX += (int)EntityPlane.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityPlane.Step <= _pictureHeight.Value - _drawingPlaneHeight)
                {
                    _startPosY += (int)EntityPlane.Step;
                }
                return true;
            default:
                return false;
        }
    }

    public virtual void DrawTransport(Graphics g)
    {
        if (EntityPlane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
        Brush bodyBrush = new SolidBrush(EntityPlane.BodyColor);
        Brush brBlack = new SolidBrush(Color.Black);

        //границы самолета
        g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value + 34, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value + 44, 20, 20);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 45, 10, 10);
        g.DrawRectangle(pen, _startPosX.Value + 170, _startPosY.Value + 45, 10, 10);
        g.DrawRectangle(pen, _startPosX.Value + 10, _startPosY.Value + 34, 160, 30);

        Point[] pointsFaceTop = {
            new Point(_startPosX.Value + 170, _startPosY.Value + 30),
            new Point(_startPosX.Value + 170, _startPosY.Value + 50),
            new Point(_startPosX.Value + 190, _startPosY.Value + 50)
        };
        g.DrawPolygon(pen, pointsFaceTop);

        Point[] pointsFaceBottom = {
            new Point(_startPosX.Value + 170, _startPosY.Value + 70),
            new Point(_startPosX.Value + 170, _startPosY.Value + 50),
            new Point(_startPosX.Value + 190, _startPosY.Value + 50)
        };
        g.DrawPolygon(pen, pointsFaceBottom);

        Point[] pointsKeel = {
            new Point(_startPosX.Value + 10, _startPosY.Value),
            new Point(_startPosX.Value + 10, _startPosY.Value + 35),
            new Point(_startPosX.Value + 50, _startPosY.Value + 35)
        };
        g.DrawPolygon(pen, pointsKeel);

        // Рисуем триммера
        g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value + 30, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 30, _startPosY.Value + 30, 10, 10);
        g.DrawRectangle(pen, _startPosX.Value + 5, _startPosY.Value + 30, 30, 10);

        // Рисуем крыло
        g.DrawEllipse(pen, _startPosX.Value + 70, _startPosY.Value + 45, 6, 6);
        g.DrawEllipse(pen, _startPosX.Value + 130, _startPosY.Value + 45, 6, 6);
        g.DrawRectangle(pen, _startPosX.Value + 75, _startPosY.Value + 45, 60, 6);

        // Закрашиваем носовую часть и киль
        g.FillPolygon(bodyBrush, pointsFaceBottom);
        g.FillPolygon(bodyBrush, pointsFaceTop);
        g.FillPolygon(bodyBrush, pointsKeel);

        // Закрашиваем корпус
        g.FillEllipse(bodyBrush, _startPosX.Value, _startPosY.Value + 34, 20, 20);
        g.FillEllipse(bodyBrush, _startPosX.Value, _startPosY.Value + 44, 20, 20);
        g.FillRectangle(bodyBrush, _startPosX.Value, _startPosY.Value + 45, 10, 10);
        g.FillRectangle(bodyBrush, _startPosX.Value + 10, _startPosY.Value + 34, 160, 30);

        // Закрашиваем триммера
        g.FillEllipse(brBlack, _startPosX.Value, _startPosY.Value + 30, 10, 10);
        g.FillEllipse(brBlack, _startPosX.Value + 30, _startPosY.Value + 30, 10, 10);
        g.FillRectangle(brBlack, _startPosX.Value + 5, _startPosY.Value + 30, 30, 10);

        // Закрашиваем крыло
        g.FillEllipse(brBlack, _startPosX.Value + 70, _startPosY.Value + 44, 6, 6);
        g.FillEllipse(brBlack, _startPosX.Value + 130, _startPosY.Value + 44, 6, 6);
        g.FillRectangle(brBlack, _startPosX.Value + 75, _startPosY.Value + 45, 60, 5);
    }
}

