﻿using ProjectSeaplane.Drawings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectSeaplane.MovementStrategy;

public class MoveablePlane : IMoveableObject
{
    private readonly DrawingPlane? _plane = null;

    public MoveablePlane(DrawingPlane plane)
    {
        _plane = plane;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_plane == null || _plane.EntityPlane == null || !_plane.GetPosX.HasValue || !_plane.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_plane.GetPosX.Value, _plane.GetPosY.Value, _plane.GetWidth, _plane.GetHeight);
        }
    }

    public int GetStep => (int)(_plane?.EntityPlane?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if (_plane == null || _plane.EntityPlane == null)
        {
            return false;
        }

        return _plane.MoveTransport(GetDirectionType(direction));
    }

    /// <summary>
    /// Конвертация из MovementDirection в DirectionType
    /// </summary>
    /// <param name="direction">MovementDirection</param>
    /// <returns>DirectionType</returns>
    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }
}
